%=========================================================================================
% TUTORIAL 10 to 'Numerical Methods for PDEs', WS 01/02
% Lecture given by Dr.C.Coclici, assisted by M.Rheinlnder
%=========================================================================================
%=========================================================================================
addpath fluxlimiter;

%fhandle  = @peak;
fhandle = @jump;

M    = 100;
alfa =  -1;
beta =   1;
a    = 0.5;
cfl  = 0.7;
dx   = (beta - alfa)/M;
dt   = dx*cfl/a;
IT   = 0;
t    = 0;

X = [alfa+dx/2:dx:beta-dx/2];
U1 = zeros(1,M);
U2 = zeros(1,M);
U3 = zeros(1,M);

%idx = inline('mod((i-1)+eval('M'),eval('M'))','i');
idx = inline('rem(i-1+M,M)+1','M','i');
phi = inline('vanLeer(r)','r');

%----------------------------------------------
% Initialization
%----------------------------------------------
U1 = feval(fhandle,alfa,beta,X);
U2 = U1; U3 = U1;
c = cfl;

%----------------------------------------------
% Time-Loop
%----------------------------------------------
while(1)
  figure(1); clf; hold on;
    Y  = alfa + mod((X-alfa) - a*t, beta-alfa);
    UE = feval(fhandle,alfa,beta,Y);
    plot([alfa-dx/2 X beta+dx/2], [U1(M) U1  U1(1)],'r-');
    plot([alfa-dx/2 X beta+dx/2], [U2(M) U2  U2(1)],'g--');
    plot([alfa-dx/2 X beta+dx/2], [U3(M) U3  U3(1)],'k--');
    plot([alfa-dx/2 X beta+dx/2], [UE(M) UE  UE(1)],'b--');
    title('High Resolution Methods');
    xlabel('x'); ylabel('u');
    text(alfa+0.03*(beta-alfa),1.45,['IT= ' int2str(IT) ]);
    legend('High Resolution','Upwind','Lax-Wendroff','exact');
    axis([alfa beta -0.5 1.5]);

  it = input('number of performing time-steps ','s');
  if(isempty(it)==1) it=1;
  elseif(it=='q') break; end;
  it = numeric(it);

  for i=1:it,
    t = t+dt; IT = IT +1;
    if(1)
    q = -6;
    for j=1:M,
      num = U1(idx(M, j )) - U1(idx(M,j-1));
      den = U1(idx(M,j+1)) - U1(idx(M, j ));
      if(    abs(den) > 10^q & abs(num) > 10^q) r1(j) = num/den;
      elseif(abs(den) < 10^q & abs(num) > 10^q) r1(j) = 0;
      elseif(abs(den) < 10^q & abs(num) < 10^q) r1(j) = 0;
      end;
      num = U1(idx(M,j-1)) - U1(idx(M,j-2));
      den = U1(idx(M, j )) - U1(idx(M,j-1));
      if(    abs(den) > 10^q  & abs(num) > 10^q ) r2(j) = num/den;
      elseif(abs(den) < 10^q  & abs(num) > 10^q ) r2(j) = 0;
      elseif(abs(den) < 10^q  & abs(num) < 10^q ) r2(j) = 0;
      end;
    end;
    end;%if
    %r1 = ( U1 - U1(idx(M,[0:99])) )./ ( U1(idx(M,[2:101])) - U1 );
    %r2 = ( U1(idx(M,[0:99])) - U1(idx(M,[-1:98])) )./ ( U1 - U1(idx(M,[0:99])) );
    p1  = phi(r1); p2  = phi(r2);
    U1  = U1 - c*( U1 - U1(idx(M,[0:99])) ) ...
          - 0.5*c*(1-c) * (  p1.*( U1(idx(M,[2:101])) - U1 ) - p2.*(  U1 - U1(idx(M,[0:99])) ) );
    U2  = U2 - c*( U2 - U2(idx(M,[0:99])) );
    U3  = U3 - 0.5*c  *( U3(idx(M,[2:101])) - U3(idx(M,[0:99])) )  ...
             + 0.5*c^2*(U3(idx(M,[2:101])) - 2*U3 + U3(idx(M,[0:99])));
  end;

  if(1)
  figure(2); clf; hold on;
    plot([alfa-dx/2 X beta+dx/2], [r1(M) r1  r1(1)],'b-');
    plot([alfa-dx/2 X beta+dx/2], [r2(M) r2  r2(1)],'r--');
    title('The Variation Ratios');
    legend('r1 = (u_j - u_{j+1}) / (u_{j+1} - u_j)','r2 = (u_{j-1} - u_{j-2}) / (u_j - u_{j-1})');
    xlabel('x');
  end;
  if(1)
  figure(3); clf; hold on;
    plot([alfa-dx/2 X beta+dx/2], [p1(M) p1  p1(1)],'b-');
    plot([alfa-dx/2 X beta+dx/2], [p2(M) p2  p2(1)],'r--');
    title('The Flux Limiter');
    legend('phi(r1)','phi(r2)');
    axis([alfa beta -1 3]);
    xlabel('x');
  end;
end; %while


